@@ -18,7 +18,7 @@ class Analyze(Ready):
     """
 
     name = "analyze"
-    allowed = []
+    allowed = ["delete"]
     exported_methods = {
         "get_averaged_cong",
         "get_congl",
@@ -26,6 +26,8 @@ class Analyze(Ready):
         "get_dcline_pf",
         "get_lmp",
         "get_load_shed",
+        "get_load_shift_up",
+        "get_load_shift_dn",
         "get_pf",
         "get_pg",
         "get_storage_e",
@@ -37,7 +39,6 @@ class Analyze(Ready):
         """Constructor."""
         super().__init__(scenario)
 
-        self.data_loc = "disk" if scenario.status == "moved" else None
         self.refresh(scenario)
 
     def refresh(self, scenario):
@@ -53,11 +54,11 @@ class Analyze(Ready):
     def _set_allowed_state(self):
         """Sets allowed state."""
         if self._scenario_status == "extracted":
-            self.allowed = ["delete", "move"]
+            self.allowed.append("move")
 
     def _set_ct_and_grid(self):
         """Sets change table and grid."""
-        input_data = InputData(data_loc=self.data_loc)
+        input_data = InputData()
         grid_mat_path = input_data.get_data(self._scenario_info, "grid")
         self.grid = Grid(
             interconnect=[None],
@@ -110,65 +111,50 @@ class Analyze(Ready):
                     )
                 )
 
+    def _get_data(self, field):
+        return OutputData().get_data(self._scenario_info["id"], field)
+
     def get_pg(self):
         """Returns PG data frame.
 
         :return: (*pandas.DataFrame*) -- data frame of power generated.
         """
-        output_data = OutputData(data_loc=self.data_loc)
-        pg = output_data.get_data(self._scenario_info["id"], "PG")
-
-        return pg
+        return self._get_data("PG")
 
     def get_pf(self):
         """Returns PF data frame.
 
         :return: (*pandas.DataFrame*) -- data frame of power flow.
         """
-        output_data = OutputData(data_loc=self.data_loc)
-        pf = output_data.get_data(self._scenario_info["id"], "PF")
-
-        return pf
+        return self._get_data("PF")
 
     def get_dcline_pf(self):
         """Returns PF_DCLINE data frame.
 
         :return: (*pandas.DataFrame*) -- data frame of power flow on DC line(s).
         """
-        output_data = OutputData(data_loc=self.data_loc)
-        dcline_pf = output_data.get_data(self._scenario_info["id"], "PF_DCLINE")
-
-        return dcline_pf
+        return self._get_data("PF_DCLINE")
 
     def get_lmp(self):
         """Returns LMP data frame. LMP = locational marginal price
 
         :return: (*pandas.DataFrame*) -- data frame of nodal prices.
         """
-        output_data = OutputData(data_loc=self.data_loc)
-        lmp = output_data.get_data(self._scenario_info["id"], "LMP")
-
-        return lmp
+        return self._get_data("LMP")
 
     def get_congu(self):
         """Returns CONGU data frame. CONGU = Congestion, Upper flow limit
 
         :return: (*pandas.DataFrame*) -- data frame of branch flow mu (upper).
         """
-        output_data = OutputData(data_loc=self.data_loc)
-        congu = output_data.get_data(self._scenario_info["id"], "CONGU")
-
-        return congu
+        return self._get_data("CONGU")
 
     def get_congl(self):
         """Returns CONGL data frame. CONGL = Congestion, Lower flow limit
 
         :return: (*pandas.DataFrame*) -- data frame of branch flow mu (lower).
         """
-        output_data = OutputData(data_loc=self.data_loc)
-        congl = output_data.get_data(self._scenario_info["id"], "CONGL")
-
-        return congl
+        return self._get_data("CONGL")
 
     def get_averaged_cong(self):
         """Returns averaged CONGL and CONGU.
@@ -176,10 +162,7 @@ class Analyze(Ready):
         :return: (*pandas.DataFrame*) -- data frame of averaged congestion with
             the branch id as indices an the averaged CONGL and CONGU as columns.
         """
-        output_data = OutputData(data_loc=self.data_loc)
-        mean_cong = output_data.get_data(self._scenario_info["id"], "AVERAGED_CONG")
-
-        return mean_cong
+        return self._get_data("AVERAGED_CONG")
 
     def get_storage_pg(self):
         """Returns STORAGE_PG data frame.
@@ -187,37 +170,30 @@ class Analyze(Ready):
         :return: (*pandas.DataFrame*) -- data frame of power generated by
             storage units.
         """
-        output_data = OutputData(data_loc=self.data_loc)
-        storage_pg = output_data.get_data(self._scenario_info["id"], "STORAGE_PG")
-
-        return storage_pg
+        return self._get_data("STORAGE_PG")
 
     def get_storage_e(self):
         """Returns STORAGE_E data frame. Energy state of charge.
 
         :return: (*pandas.DataFrame*) -- data frame of energy state of charge.
         """
-        output_data = OutputData(data_loc=self.data_loc)
-        storage_e = output_data.get_data(self._scenario_info["id"], "STORAGE_E")
-
-        return storage_e
+        return self._get_data("STORAGE_E")
 
     def get_load_shed(self):
         """Returns LOAD_SHED data frame, either via loading or calculating.
 
         :return: (*pandas.DataFrame*) -- data frame of load shed (hour x bus).
         """
-        scenario_id = self._scenario_info["id"]
         try:
             # It's either on the server or in our local ScenarioData folder
-            output_data = OutputData(data_loc=self.data_loc)
-            load_shed = output_data.get_data(scenario_id, "LOAD_SHED")
+            load_shed = self._get_data("LOAD_SHED")
         except OSError:
             # The scenario was run without load_shed, and we must construct it
             grid = self.get_grid()
             infeasibilities = self._parse_infeasibilities()
             load_shed = construct_load_shed(self._scenario_info, grid, infeasibilities)
 
+            scenario_id = self._scenario_info["id"]
             filename = scenario_id + "_LOAD_SHED.pkl"
             output_dir = server_setup.OUTPUT_DIR
             filepath = os.path.join(server_setup.LOCAL_DIR, output_dir, filename)
@@ -226,6 +202,23 @@ class Analyze(Ready):
 
         return load_shed
 
+        def get_load_shift_up(self):
+            """Returns LOAD_SHIFT_UP data frame. This is the amount that flexible demand
+            deviates above (e.g., recovers) the base demand.
+
+            :return: (*pandas.DataFrame*) -- data frame of load shifted up (hour x bus).
+            """
+            return self._get_data("LOAD_SHIFT_UP")
+
+        def get_load_shift_dn(self):
+            """Returns LOAD_SHIFT_DN data frame. This is the amount that flexible demand
+            deviates below (e.g., curtails) the base demand.
+
+            :return: (*pandas.DataFrame*) -- data frame of load shifted down (hour x
+                bus).
+            """
+            return self._get_data("LOAD_SHIFT_DN")
+
     def get_demand(self, original=True):
         """Returns demand profiles.
 
