@@ -1,12 +1,13 @@
-import os
+import pandas as pd
+from scipy.io import savemat
 
 from powersimdata.data_access.context import Context
-from powersimdata.input.export_data import export_case_mat, export_transformed_profile
+from powersimdata.input.export_data import export_case_mat
 from powersimdata.input.grid import Grid
 from powersimdata.input.input_data import InputData
 from powersimdata.input.transform_grid import TransformGrid
+from powersimdata.input.transform_profile import TransformProfile
 from powersimdata.scenario.ready import Ready
-from powersimdata.utility import server_setup
 from powersimdata.utility.config import get_deployment_mode
 
 
@@ -17,7 +18,7 @@ class Execute(Ready):
     """
 
     name = "execute"
-    allowed = []
+    allowed = ["delete"]
     exported_methods = {
         "check_progress",
         "extract_simulation_output",
@@ -103,12 +104,20 @@ class Execute(Ready):
             si = SimulationInput(
                 self._data_access, self._scenario_info, self.grid, self.ct
             )
-            si.create_folder()
             for p in ["demand", "hydro", "solar", "wind"]:
                 si.prepare_profile(p, profiles_as)
 
             si.prepare_mpc_file()
 
+            if "demand_flexibility" in self.ct:
+                # Prepare all specified demand flexibility profiles
+                for p in list(self.ct["demand_flexibility"]):
+                    if p != "demand_flexibility_duration":
+                        si.prepare_profile(p, profiles_as)
+
+                # Create the demand_flexibility_parameters file
+                si.prepare_demand_flexibility_parameters_file()
+
             prepared = "prepared"
             self._execute_list_manager.set_status(self.scenario_id, prepared)
             self._scenario_status = prepared
@@ -195,45 +204,66 @@ class SimulationInput:
 
         self.REL_TMP_DIR = self._data_access.tmp_folder(self.scenario_id)
 
-    def create_folder(self):
-        """Creates folder on server that will enclose simulation inputs."""
-        description = self._data_access.description
-        print(f"--> Creating temporary folder on {description} for simulation inputs")
-        self._data_access.makedir(self.REL_TMP_DIR)
-
     def prepare_mpc_file(self):
         """Creates MATPOWER case file."""
-        file_name = f"{self.scenario_id}_case.mat"
-        storage_file_name = f"{self.scenario_id}_case_storage.mat"
-        file_path = os.path.join(server_setup.LOCAL_DIR, file_name)
-        storage_file_path = os.path.join(server_setup.LOCAL_DIR, storage_file_name)
-        print("Building MPC file")
-        export_case_mat(self.grid, file_path, storage_file_path)
-        self._data_access.move_to(
-            file_name, self.REL_TMP_DIR, change_name_to="case.mat"
-        )
-        if len(self.grid.storage["gen"]) > 0:
-            self._data_access.move_to(
-                storage_file_name, self.REL_TMP_DIR, change_name_to="case_storage.mat"
-            )
+        file_path = "/".join([self.REL_TMP_DIR, "case.mat"])
+        storage_file_path = "/".join([self.REL_TMP_DIR, "case_storage.mat"])
 
-    def prepare_profile(self, kind, profile_as=None):
+        print("Building MPC file")
+        mpc, mpc_storage = export_case_mat(self.grid)
+
+        with self._data_access.write(file_path, save_local=False) as f:
+            savemat(f, mpc, appendmat=False)
+
+        if mpc_storage is not None:
+            with self._data_access.write(storage_file_path, save_local=False) as f:
+                savemat(f, mpc, appendmat=False)
+
+    def prepare_profile(self, kind, profile_as=None, slice=False):
         """Prepares profile for simulation.
 
-        :param kind: one of *demand*, *'hydro'*, *'solar'* or *'wind'*.
+        :param kind: one of *demand*, *'hydro'*, *'solar'*, *'wind'*,
+            *'demand_flexibility_up'*, *'demand_flexibility_dn'*,
+            *'demand_flexibility_cost_up'*, or *'demand_flexibility_cost_dn'*.
         :param int/str profile_as: if given, copy profile from this scenario.
+        :param bool slice: whether to slice the profiles by the Scenario's time range.
         """
+        file_name = f"{kind}.csv"
         if profile_as is None:
-            file_name = "%s_%s.csv" % (self.scenario_id, kind)
-            filepath = os.path.join(server_setup.LOCAL_DIR, file_name)
-            export_transformed_profile(
-                kind, self._scenario_info, self.grid, self.ct, filepath
-            )
+            filepath = "/".join([self.REL_TMP_DIR, file_name])
 
-            self._data_access.move_to(
-                file_name, self.REL_TMP_DIR, change_name_to=f"{kind}.csv"
-            )
+            tp = TransformProfile(self._scenario_info, self.grid, self.ct, slice)
+            profile = tp.get_profile(kind)
+            print(f"Writing scaled {kind} profile to {filepath}")
+            with self._data_access.write(filepath, save_local=False) as f:
+                profile.to_csv(f)
         else:
             from_dir = self._data_access.tmp_folder(profile_as)
-            src = self._data_access.join(from_dir, f"{kind}.csv")
+            src = "/".join([from_dir, file_name])
             self._data_access.copy(src, self.REL_TMP_DIR)
+
+    def prepare_demand_flexibility_parameters_file(self):
+        """Creates the demand_flexibility_parameters file."""
+        # Construct a dictionary of the necessary parameters
+        print("Building demand flexibility parameters file")
+        params = {}
+        params["enabled"] = [True]
+        params["interval_balance"] = [True]
+        params["rolling_balance"] = [
+            True
+            if "demand_flexibility_duration" in self.ct["demand_flexibility"]
+            else False
+        ]
+        params["duration"] = [
+            self.ct["demand_flexibility"]["demand_flexibility_duration"]
+            if "demand_flexibility_duration" in self.ct["demand_flexibility"]
+            else 0
+        ]
+
+        # Convert the dictionary to a DataFrame to more easily save as a .csv file
+        params = pd.DataFrame.from_dict(params)
+
+        # Save the parameters as a .csv file and move the file to the correct location
+        file_path = "/".join([self.REL_TMP_DIR, "demand_flexibility_parameters.csv"])
+        with self._data_access.write(file_path, save_local=False) as f:
+            params.to_csv(f)
